package com.aitrainer.api.controller.diet

import com.aallam.openai.api.BetaOpenAI
import com.aallam.openai.api.chat.ChatMessage
import com.aallam.openai.api.chat.ChatRole
import com.aitrainer.api.model.OpenAIChat
import com.aitrainer.api.model.diet.Diet
import com.aitrainer.api.openai.OpenAIService
import com.aitrainer.api.repository.diet.DietRepository
import com.google.gson.Gson
import kotlinx.coroutines.DelicateCoroutinesApi
import kotlinx.coroutines.GlobalScope
import kotlinx.coroutines.async
import kotlinx.coroutines.runBlocking
import org.slf4j.LoggerFactory
import org.springframework.beans.factory.annotation.Value
import org.springframework.http.MediaType
import org.springframework.http.ResponseEntity
import org.springframework.web.bind.annotation.*
import java.time.LocalDateTime
import java.time.format.DateTimeFormatter
import java.util.*

@RestController
@RequestMapping("/api")
class DietController(private val dietRepository: DietRepository) {
    private val logger = LoggerFactory.getLogger(javaClass)

    @PostMapping("/diet", produces = [MediaType.APPLICATION_JSON_VALUE + ";charset=UTF-8"])
    fun insert(@RequestBody diet: Diet): ResponseEntity<*> {
        return ResponseEntity.ok().body(dietRepository.save(diet))
    }

    @PostMapping("/diet/{id}", produces = [MediaType.APPLICATION_JSON_VALUE + ";charset=UTF-8"])
    fun update(@PathVariable(value = "id") id: Long, @RequestBody diet: Diet): ResponseEntity<Diet> {
        val existingDiet  = dietRepository.findByDietId(id) ?: return ResponseEntity.notFound().build()

        val updatedDiet: Diet = existingDiet.copy(
            dietId = diet.dietId,
            dietUserId = diet.dietUserId,
            dietText = diet.dietText,
            startDate = diet.startDate
        )
        diet.meals.forEach {
            it.diet = diet
            updatedDiet.meals.add(it)
        }
        return ResponseEntity.ok().body(dietRepository.save(updatedDiet))
    }

    @PostMapping("/diet/no_text/{id}", produces = [MediaType.APPLICATION_JSON_VALUE + ";charset=UTF-8"])
    fun updateNoText(@PathVariable(value = "id") id: Long, @RequestBody diet: Diet): ResponseEntity<Diet> {
        val existingDiet  = dietRepository.findByDietId(id) ?: return ResponseEntity.notFound().build()

        val updatedDiet: Diet = existingDiet.copy(
            dietId = diet.dietId,
            dietUserId = diet.dietUserId,
            startDate = diet.startDate
        )
        diet.meals.forEach {
            it.diet = diet
            updatedDiet.meals.add(it)
        }
        return ResponseEntity.ok().body(dietRepository.save(updatedDiet))
    }

    @GetMapping("/diet/{dietUserId}", produces = [MediaType.APPLICATION_JSON_VALUE + ";charset=UTF-8"])
    fun getByDietUserId(@PathVariable dietUserId: Long): ResponseEntity<List<Diet>> {
        val list = dietRepository.findByDietUserId(dietUserId)

        return if (list.isEmpty()) ResponseEntity.notFound().build() else
            ResponseEntity.ok().body(list)
    }

    @OptIn(BetaOpenAI::class, DelicateCoroutinesApi::class)
    @PostMapping("/diet/generate_premium/{dietUserId}", produces = [MediaType.APPLICATION_JSON_VALUE + ";charset=UTF-8"])
    fun generatePremiumDiet(@PathVariable dietUserId: Long, @RequestBody input: String, @Value("\${openai.key}") openaiKey: String): ResponseEntity<*> {

        val now = LocalDateTime.now()
        val formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss")
        val todayString = now.format(formatter)
        var diet: Diet? = dietRepository.findTheLastEmptyDiet(dietUserId)
        if ( diet == null ) {
            diet = Diet(
                dietUserId = dietUserId,
                dietText = "",
                startDate = todayString,
                premium = 0
            )
            diet = dietRepository.save(diet)
        }
        val systemMsg = ChatMessage(
            role = ChatRole.System,
            content = "Te egy táplálkozási szakértő vagy. A feladat mindig egy étrend elkészítése ebben a 3 lépésben:\n" +
                    "\n" +
                    "1. Vizsgáld meg az input adatokat, és készítsd el az étrendet\n" +
                    "2. Ellenőrizd az ételeket, hogy tényleg létezik-e olyan, és megfelel az input adatok feltételeinek\n" +
                    "3. Ellenőrizd a kimeneti formátumot. Csak a megadott kimeneti formátumban válaszolj"
        )
        val userMsg = ChatMessage(
            role = ChatRole.User,
            content = input
        )
        val listMessages: MutableList<ChatMessage> = mutableListOf()
        listMessages.add(systemMsg)
        listMessages.add(userMsg)
        val gson = Gson()
        val messages = gson.toJson(listMessages)

        val openai = OpenAIChat(
            messages = messages,
            //modelName = "gpt-3.5-turbo",
            modelName = "gpt-4",
            temperature = 0.9
        )

        var result: String? = null
        val openAIService = OpenAIService(openaiKey, openai.modelName, openai.temperature)
        val deferred = GlobalScope.async {
            println(openai.messages)
            openAIService.chatCompletion(openai.messages)
        }
        runBlocking {
            try {
                result = deferred.await().toString()
            } catch (exception: Exception) {
                println("Timeout for diet generation $dietUserId")
            }

        }
        if ( result == null ) {
            return ResponseEntity.badRequest().build<String>()
        }

        diet.dietText = result!!
        diet.startDate = todayString
        diet.premium = 1
        return ResponseEntity.ok().body(dietRepository.save(diet))
    }
}