package com.aitrainer.api.service

import com.aitrainer.api.model.firebase_response.SignupResponse
import com.google.gson.Gson
import org.json.JSONObject
import org.springframework.beans.factory.annotation.Value
import org.springframework.stereotype.Service
import java.net.URI
import java.net.http.HttpClient
import java.net.http.HttpRequest
import java.net.http.HttpResponse
import java.nio.charset.StandardCharsets
import java.util.*

@Service
class Firebase(@Value("\${firebase.key}") private val apiKey: java.lang.String) {
    private val firebaseBaseUrl: String = "https://identitytoolkit.googleapis.com/v1/accounts"
    var statusCode: Int = 0
    var error: String = ""

    fun signUp(email: String, password: String): SignupResponse? {

        val json = JSONObject()
            .put("email", email)
            .put("password", password)
            .put("returnSecureToken", "true")

        // Create the POST body with the email and password
        val postData = json.toString()
        println("Firebase post: $postData")

        val response = this.call("signUp", postData)
        if ( statusCode != 200) {
            return null
        }

        val gson = Gson()
        return gson.fromJson(response, SignupResponse::class.java)
    }

    fun deleteUser(idToken: String): Int {
        val json = JSONObject().put("idToken", idToken)
        val postData = json.toString()
        this.call("delete", postData)
        return statusCode
    }

    private fun call(command: String, postData: String): String {
        val apiKey = this.apiKey
        val url = URI.create("$firebaseBaseUrl:$command?key=$apiKey")

        // Create the HTTP request object
        val request = HttpRequest.newBuilder()
            .POST(HttpRequest.BodyPublishers.ofString(postData))
            .uri(url)
            .header("Content-Type", "application/json")
            .build()

        // Send the request and get the response
        val response = HttpClient.newBuilder().build().send(request, HttpResponse.BodyHandlers.ofString(StandardCharsets.UTF_8))
        statusCode = response.statusCode()

        if ( statusCode != 200) {
            error = JSONObject(response.body()).getJSONObject("error").getString("message").toString()
            return error
        }
        error = ""
        return response.body()
    }
}